pub mod auth;
pub mod axum;
pub mod introspection;
pub mod login;
pub mod oauth;
pub mod session;

#[cfg(test)]
mod test {
    use clap::Parser;
    use pretty_assertions::assert_eq;

    use crate::{
        login::{
            config::{AuthEngine, ZitadelClap},
            AuthClap,
        },
        session::{PostgresqlSessionClap, SessionBackend, SessionClap},
    };

    #[derive(Parser)]
    #[command(author, version, about, long_about = None)]
    pub struct Cli {
        #[command(subcommand)]
        command: Commands,
    }

    #[derive(clap::Subcommand, Clone, Debug, Eq, PartialEq)]
    pub enum Commands {
        One {
            #[clap(flatten)]
            options: AuthClap,
        },
    }

    #[test]
    fn test_command_parse_as_default_noop() {
        let cli: Cli = Cli::parse_from(["base", "one"]);

        assert_eq!(
            cli.command,
            Commands::One {
                options: AuthClap {
                    engine: AuthEngine::Noop,
                    zitadel: ZitadelClap {
                        client_id: None,
                        client_secret: None,
                        redirect_url: None,
                        authority_url: None,
                    },
                    session_backend: SessionBackend::InMemory,
                    session: SessionClap {
                        postgresql: PostgresqlSessionClap { conn: None }
                    }
                }
            }
        );
    }

    #[test]
    fn test_command_parse_as_noop() {
        let cli: Cli = Cli::parse_from(["base", "one", "--auth-engine", "noop"]);

        assert_eq!(
            cli.command,
            Commands::One {
                options: AuthClap {
                    engine: AuthEngine::Noop,
                    zitadel: ZitadelClap {
                        client_id: None,
                        client_secret: None,
                        redirect_url: None,
                        authority_url: None,
                    },
                    session_backend: SessionBackend::InMemory,
                    session: SessionClap {
                        postgresql: PostgresqlSessionClap { conn: None }
                    }
                }
            }
        );
    }

    #[test]
    fn test_command_parse_as_zitadel() {
        let cli: Cli = Cli::parse_from([
            "base",
            "one",
            "--auth-engine=zitadel",
            "--zitadel-client-id=something",
            "--zitadel-client-secret=something",
            "--zitadel-auth-url=https://something",
            "--zitadel-redirect-url=https://something",
            "--zitadel-token-url=https://something",
            "--zitadel-authority-url=https://something",
        ]);

        assert_eq!(
            cli.command,
            Commands::One {
                options: AuthClap {
                    engine: AuthEngine::Zitadel,
                    zitadel: ZitadelClap {
                        client_id: Some("something".into()),
                        client_secret: Some("something".into()),
                        redirect_url: Some("https://something".into()),
                        authority_url: Some("https://something".into()),
                    },
                    session_backend: SessionBackend::InMemory,
                    session: SessionClap {
                        postgresql: PostgresqlSessionClap { conn: None }
                    }
                },
            }
        );
    }
}
