use std::{ops::Deref, sync::Arc};

use async_trait::async_trait;

use oauth2::url::Url;

use self::{
    noop::NoopOAuthClient,
    zitadel::{ZitadelConfig, ZitadelOAuthClient},
};

pub struct OAuth(Arc<dyn OAuthClient + Send + Sync + 'static>);

impl OAuth {
    pub fn new_zitadel(config: ZitadelConfig) -> Self {
        Self(Arc::new(ZitadelOAuthClient::from(config)))
    }
    pub fn new_noop() -> Self {
        Self(Arc::new(NoopOAuthClient {}))
    }
}

impl Deref for OAuth {
    type Target = Arc<dyn OAuthClient + Send + Sync + 'static>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

#[async_trait]
pub trait OAuthClient {
    async fn get_token(&self) -> anyhow::Result<()>;
    async fn authorize_url(&self) -> anyhow::Result<Url>;
    async fn exchange(&self, code: &str) -> anyhow::Result<String>;
}

pub mod noop;
pub mod zitadel;

#[cfg(test)]
mod tests {
    use clap::Parser;
    use sealed_test::prelude::*;

    use crate::login::config::ZitadelClap;

    #[derive(Parser)]
    #[command(author, version, about, long_about = None)]
    pub struct Cli {
        #[clap(flatten)]
        options: ZitadelClap,
    }

    #[derive(Parser, Debug)]
    #[command(author, version, about, long_about = None)]
    pub struct CliSubCommand {
        #[command(subcommand)]
        command: Commands,
    }

    #[derive(clap::Subcommand, Clone, Debug, Eq, PartialEq)]
    pub enum Commands {
        One {
            #[clap(flatten)]
            options: ZitadelClap,
        },
    }

    #[tokio::test]
    async fn test_parse_clap_zitadel() {
        let cli: Cli = Cli::parse_from([
            "base",
            "--zitadel-client-id=something",
            "--zitadel-client-secret=something",
            "--zitadel-redirect-url=https://something",
            "--zitadel-authority-url=https://something",
        ]);
        println!("{:?}", cli.options);

        pretty_assertions::assert_eq!(
            cli.options,
            ZitadelClap {
                client_id: Some("something".into()),
                client_secret: Some("something".into()),
                redirect_url: Some("https://something".into()),
                authority_url: Some("https://something".into()),
            }
        );
    }

    #[test]
    fn test_parse_clap_zitadel_fails_require_all() {
        let cli = CliSubCommand::try_parse_from([
            "base",
            "one",
            // "--zitadel-client-id=something", // We want to trigger missing variable
            "--zitadel-client-secret=something",
            "--zitadel-redirect-url=https://something",
            "--zitadel-authority-url=https://something",
        ]);

        pretty_assertions::assert_eq!(cli.is_err(), true);
    }

    #[sealed_test]
    fn test_parse_clap_env_zitadel() {
        std::env::set_var("ZITADEL_CLIENT_ID", "something");
        std::env::set_var("ZITADEL_CLIENT_SECRET", "something");
        std::env::set_var("ZITADEL_REDIRECT_URL", "https://something");
        std::env::set_var("ZITADEL_AUTHORITY_URL", "https://something");

        let cli = CliSubCommand::parse_from(["base", "one"]);

        pretty_assertions::assert_eq!(
            cli.command,
            Commands::One {
                options: ZitadelClap {
                    client_id: Some("something".into()),
                    client_secret: Some("something".into()),
                    redirect_url: Some("https://something".into()),
                    authority_url: Some("https://something".into()),
                }
            }
        );
    }
    #[test]
    fn test_parse_clap_defaults_to_noop() {
        let cli = CliSubCommand::parse_from(["base", "one"]);

        pretty_assertions::assert_eq!(
            cli.command,
            Commands::One {
                options: ZitadelClap {
                    client_id: None,
                    client_secret: None,
                    redirect_url: None,
                    authority_url: None,
                },
            }
        );
    }
}
